from django.conf.urls import include, patterns, url


urlpatterns = patterns('',
                       url(r'^packages/', include('leonardo_system.package.urls')),
                       url(r'^maintenance/', include('leonardo_system.maintenance.urls')),
                       )



from django.utils.translation import ugettext_lazy as _
from django_select2.fields import AutoSelect2TagField
from django_select2.views import NO_ERR_RESP
from leonardo import forms, messages

from .utils import pip_install, update_all


class PluginSelectField(AutoSelect2TagField):

    """returns list of plugins from github group or pypi

    it's simple PoC !
    """

    search_fields = ['tag__icontains', ]

    def get_field_values(self, value):
        raise Exception(value)
        return {'tag': value}

    def get_results(self, request, term, page, context):

        pkgs = update_all()

        res = [
            (
                repo.name,
                repo.name,
                {}
            )
            for repo in pkgs if term in repo.name
        ]

        return NO_ERR_RESP, True, res


class PluginInstallForm(forms.SelfHandlingForm):
    """simple form for installing packages

    this support new abilities like an dynamic plugin install etc..
    """

    packages = PluginSelectField(label=_('Search packages'))

    reload_server = forms.BooleanField(
        label=_('Reload Server'), initial=False,
        required=False,
        help_text=_('Warning: this kill this Leonardo instance !!!\
                    For successfull reload must be run under Supervisor !\
                    You may lost your data !'),)

    def __init__(self, *args, **kwargs):
        super(PluginInstallForm, self).__init__(*args, **kwargs)

        self.helper.layout = forms.Layout(
            forms.Accordion('', 'packages',
                            forms.AccordionGroup(
                                _('Advanced options'),
                                'reload_server',
                                )
                            ),
        )

    def handle(self, request, data):

        kwargs = data
        kwargs['request'] = request
        try:
            pip_install(**kwargs)
        except Exception as e:
            messages.error(request, str(e))
        else:
            return True
        return False




from django.utils.translation import ugettext_lazy as _
from leonardo import messages
from leonardo.utils import dotdict

from django.core import management

try:
    import github
    GITHUB = True
except Exception:
    GITHUB = False
# github specific
ORG = None

try:
    import pip  # noqa
    PIP = True
except Exception:
    PIP = False


# for this time
# all items for fast search
# in memory ..
REPOS = None
GITHUB_REPOS = None

AVAIL_WIDGETS = None
# hide from search plugin
_BLACKLIST = [
    'django-leonardo',
    'raspiface',
]


def pip_install(packages, request=None, reload_server=False):
    """install packages from pip

    if request is provided, user messages is pushed out
    """

    if PIP:

        # install
        try:
            pip.main(['install'] + list(packages))
            if request:
                messages.success(request, _(
                    'Packages %s was successfully installed,\
                     please restart your server.' % list(packages)))
        except Exception as e:
            if request:
                messages.error(request, _(
                    'Installing packages raised exception %s' % e))
            else:
                raise e
        else:
            if reload_server:
                # try self kill
                try:
                    import os
                    os.kill(os.getpid(), 9)
                except Exception as e:
                    if request:
                        messages.error(request, _(
                            'Run post install task fails with %s' % e))
                    else:
                        raise e

    else:
        if request:
            messages.error(request, _(
                'For this functionality please install pip package'))


def filter_repos():
    """filter repos for duplicity
    """

    global REPOS

    _pkgs = []
    _pkgs_names = []

    if REPOS:

        for item in REPOS:
            if item.name not in _BLACKLIST and item.name not in _pkgs_names:
                _pkgs_names.append(item.name)
                _pkgs.append(item)

        REPOS = _pkgs


def update_global_repos(items):
    """update repos safety
    """

    global REPOS

    if REPOS:
        REPOS += items
    else:
        REPOS = items


def get_widgets(query='leonardo', request=None):
    """returns all widgets
    """
    widgets = {}

    global GITHUB_REPOS

    if GITHUB_REPOS is None:
        update_packages_from_github(query, request)

    if GITHUB:

        for repo in GITHUB_REPOS:

            # TODO read setup.py
            mod_name = repo.name
            raise Exception(mod_name)
            try:
                descriptor = repo.get_file_contents(
                    '%s/__init__.py' % mod_name).decoded_content
            except Exception:
                pass
            else:
                widgets.update({
                    mod_name: descriptor.widgets
                })

        raise Exception(widgets)


def update_packages_from_pip(query, request=None):
    """updates global REPOS from pypi
    """

    if PIP:
        try:
            from pip.commands.search import SearchCommand
            search_results = SearchCommand().search(
                'leonardo', 'https://pypi.python.org/pypi')
            # serialize to dot access dict
            search_results = [dotdict(d) for d in search_results]
        except ImportError:
            if request:
                messages.error(request, _(
                    'For this functionality please install pip package'))
        else:
            update_global_repos(search_results)


def update_packages_from_github(query, request=None):
    """updates global REPOS from leonardo github organization
    """

    global REPOS
    global GITHUB_REPOS

    if GITHUB:
        global ORG

        try:
            g = github.Github()
            org = g.get_organization('leonardo-modules')
        except ImportError:
            if request:
                messages.error(request, _(
                    'For this functionality please run pip install PyGithub'))
        else:
            GITHUB_REPOS = org.get_repos()
            update_global_repos(list(org.get_repos()))
            ORG = org


def update_all(query='leonardo', request=None):

    global REPOS

    if not REPOS:
        update_packages_from_pip(query, request)
        update_packages_from_github(query, request)
        filter_repos()
    return REPOS




from __future__ import absolute_import

from django.utils.translation import ugettext_lazy as _
from leonardo.views import *
from .forms import ServerReloadForm, ManagementForm, InfoForm


class ServerReloadView(ModalFormView, ContextMixin, ModelFormMixin):

    form_class = ServerReloadForm

    def get_success_url(self):
        return self.request.build_absolute_uri()

    def get_context_data(self, **kwargs):
        context = super(ServerReloadView, self).get_context_data(**kwargs)

        context['url'] = self.request.build_absolute_uri()
        context['modal_header'] = 'Reload Server'
        context['title'] = 'Reload Server'
        context['form_submit'] = _('Submit Reload')
        context['heading'] = 'Reload Server'
        context['modal_size'] = 'sm'
        return context


class InfoView(ModalFormView, ContextMixin, ModelFormMixin):

    form_class = InfoForm

    def get_success_url(self):
        return self.request.build_absolute_uri()

    def get_form(self, form_class):
        """Returns an instance of the form to be used in this view."""

        kwargs = self.get_form_kwargs()
        kwargs.update({
            'request': self.request,
        })
        return form_class(**kwargs)

    def get_context_data(self, **kwargs):
        context = super(InfoView, self).get_context_data(**kwargs)

        context['url'] = self.request.build_absolute_uri()
        context['modal_header'] = _('Leonardo Info')
        context['title'] = _('Leonardo Info')
        context['form_submit'] = _('Close')
        context['heading'] = _('Leonardo Info')
        context['modal_size'] = 'fullscreen'
        return context

    def form_invalid(self, form):
        raise Exception(form.errors)


class ManagementView(ModalFormView, ContextMixin, ModelFormMixin):

    form_class = ManagementForm

    def get_success_url(self):
        return self.request.build_absolute_uri()

    def get_context_data(self, **kwargs):
        context = super(ManagementView, self).get_context_data(**kwargs)

        context['url'] = self.request.build_absolute_uri()
        context['modal_header'] = _('Management commands')
        context['title'] = _('Management commands')
        context['form_submit'] = _('Run commands')
        context['heading'] = _('Management commands')
        context['modal_size'] = 'md'
        return context

    def form_invalid(self, form):
        raise Exception(form.errors)



import os

from django.conf import settings
import django.dispatch
from django.core import management
from django.utils import six
from django.utils.translation import ugettext_lazy as _
from django.views.debug import get_safe_settings
from horizon import tables
from horizon_contrib.tables import FilterAction
from leonardo import forms, messages
from leonardo.utils import get_conf_from_module

server_restart = django.dispatch.Signal(providing_args=["request", "delay"])


def server_restart_callback(request, delay=0, **kwargs):

    # time.sleep(delay)
    try:

        # from django.utils.autoreload import restart_with_reloader, reloader_thread
        # kill self
        os.kill(os.getpid(), 9)

        messages.success(
            request, _('Server was successfuly restarted !'))
    except Exception as e:
        messages.error(request, str(e))
    else:
        return True
    return False


server_restart.connect(server_restart_callback)


class ServerReloadForm(forms.SelfHandlingForm):

    delay = forms.IntegerField(
        label=_('delay before restart'), initial=10, help_text=_('Delay before restart'))

    def handle(self, request, data):
        """PoC for self restart

        this support new abilities like an dynamic plugin install etc..
        """

        try:
            server_restart.send(sender=self.__class__,
                                request=request, delay=data['delay'])
            messages.warning(
                request, _('Server going to down !'))
        except Exception as e:
            messages.error(request, str(e))
        else:
            return True
        return False


class ManagementForm(forms.SelfHandlingForm):

    """form wich handle managemenet commands

    this supports running management commands via admin
    """

    makemigrations = forms.BooleanField(
        label=_('Make migrations'), initial=False,
        required=False,
        help_text=_('Run makemigrations after install ?'))

    migrate = forms.BooleanField(
        label=_('Migrate'), initial=False,
        required=False,
        help_text=_('Run migrate command after install ?'))

    sync_all = forms.BooleanField(
        label=_('Sync All'), initial=False,
        required=False,
        help_text=_('Run Sync All command after install ?'))

    sync_force = forms.BooleanField(
        label=_('Sync all force'), initial=False,
        required=False,
        help_text=_('Warning: this may override you database changes !'),)

    reload_server = forms.BooleanField(
        label=_('Reload Server'), initial=False,
        required=False,
        help_text=_('Warning: this kill this Leonardo instance !!!\
                    For successfull reload must be run under Supervisor !\
                    You may lost your data !'),)

    def __init__(self, *args, **kwargs):
        super(ManagementForm, self).__init__(*args, **kwargs)

        self.helper.layout = forms.Layout(
            forms.Accordion('',
                            'makemigrations',
                            'migrate',
                            'sync_all',
                            forms.AccordionGroup(
                                _('Advanced options'),
                                'sync_force',
                                'reload_server',
                            )
                            ),
        )

    def handle(self, request, data):

        try:
            if data.get('makemigrations', None):
                management.call_command(
                    'makemigrations', verbosity=1, interactive=False)
            if data.get('migrate', None):
                management.call_command(
                    'migrate', verbosity=1, interactive=False)
            if data.get('sync_all', None):
                management.call_command(
                    'sync_all', force=data.get('sync_force', False))
            if data.get('reload_server', None):
                import os
                os.kill(os.getpid(), 9)
        except Exception as e:
            messages.error(request, str(e))
        else:
            return True
        return False


class SettingsTable(tables.DataTable):

    key = tables.Column('key')
    value = tables.Column('value')

    def get_object_id(self, datum):
        return datum['key']

    class Meta:
        name = 'settings'
        table_actions = (FilterAction,)

PRETTY = """
{% if short %}
<div class="codeblock" tabindex="-1">
    <pre lang="json" class="short">{{ short }}</pre>
    <pre lang="json" class="full">{{ full }}</pre>
{% else %}
<div class="codeblock">
    <pre lang="json" class="short">{{ full }}</pre>
{% endif %}
</div>
"""

def prettyprint(x):
    short = None
    full = json.dumps(json.loads(x), indent=4, ensure_ascii=False)

    lines = full.split('\n')

    if (len(lines) > 5):
        short = '\n'.join(lines[:5] + ['...'])

    return render_to_string(PRETTY,
                            {"full": full, "short": short})


class LeonardoTable(tables.DataTable):

    name = tables.Column('name')
    widgets = tables.Column('config', verbose_name='Widgets', filters=(lambda c: ', '.join(c.widgets),))
    live_config = tables.Column('config', filters=(lambda c: c.config, prettyprint,))

    def get_object_id(self, datum):
        return datum['name']

    class Meta:
        name = 'leonardo-modules'
        verbose_name = _('Leonardo Modules')
        table_actions = (FilterAction,)


class InfoForm(forms.SelfHandlingForm):

    """wrapper for system info
    """

    def __init__(self, *args, **kwargs):
        request = kwargs.pop('request', None)
        super(InfoForm, self).__init__(*args, **kwargs)

        _settings = [{'key': k, 'value': v}
                     for k, v in six.iteritems(get_safe_settings())]

        table = SettingsTable(request, data=_settings)

        module_data = []

        for mod in getattr(settings, '_APPS', []):
            mod_cfg = get_conf_from_module(mod)
            module_data.append({
                'name': mod,
                'config': mod_cfg,
            })
        leonardo_table = LeonardoTable(request, data=module_data)

        self.helper.layout = forms.Layout(
            forms.TabHolder(
                forms.Tab('Settings',
                          forms.HTML(table.render())
                          ),
                forms.Tab('Leonardo modules',
                          forms.HTML(leonardo_table.render())
                          )
            )
        )

    def handle(self, request, data):
        return True



from __future__ import unicode_literals

from optparse import make_option

from django.conf import settings
from ._utils import get_versions, pp

from django.core.management.base import BaseCommand, NoArgsCommand


class Command(BaseCommand):

    help = "Check version of system packages"
    option_list = NoArgsCommand.option_list + (
        make_option('--leonardo',
                    action='store_false', dest='interactive', default=True,
                    help="Check just leonardo packages"),

        make_option('--noinput',
                    action='store_false', dest='interactive', default=True,
                    help="Do NOT prompt the user for input of any kind."),
    )

    def handle(self, **options):

        result = get_versions(settings.INSTALLED_APPS)

        output = pp.pprint(result)

        self.stdout.write(str(output))

        return "Needs System Update"



from django.conf.urls import include, patterns, url

from .views import PluginInstallView

urlpatterns = patterns('',
                       url(r'^plugin-install/$',
                           PluginInstallView.as_view(), name='plugin_install'),
                       )



from django.conf.urls import include, patterns, url

from .views import ServerReloadView, ManagementView, InfoView

urlpatterns = patterns('',
                       url(r'^server-reload/$',
                           ServerReloadView.as_view(), name='server_reload'),
                       url(r'^management-commands/$',
                           ManagementView.as_view(), name='server_management'),
                       url(r'^system-info/$',
                           InfoView.as_view(), name='system_info'),
                       )




from django.apps import AppConfig


default_app_config = 'leonardo_system.Config'


LEONARDO_APPS = ['leonardo_system']
LEONARDO_OPTGROUP = 'System'
LEONARDO_URLS_CONF = 'leonardo_system.urls'
LEONARDO_MODULE_ACTIONS = ['system/module_actions.html']
LEONARDO_ORDERING = 150


class Config(AppConfig):
    name = 'leonardo_system'
    verbose_name = "Leonardo System Module"




from __future__ import absolute_import

from django.utils.translation import ugettext_lazy as _
from leonardo.views import *
from .forms import PluginInstallForm


class PluginInstallView(ModalFormView, ContextMixin, ModelFormMixin):

    form_class = PluginInstallForm

    def get_success_url(self):
        return self.request.build_absolute_uri()

    def get_context_data(self, **kwargs):
        context = super(PluginInstallView, self).get_context_data(**kwargs)

        context['url'] = self.request.build_absolute_uri()
        context['modal_header'] = _('Install Packages')
        context['title'] = _('Install')
        context['form_submit'] = _('Install')
        context['heading'] = _('Install Packages')
        return context

    def form_invalid(self, form):
        raise Exception(form.errors)
